"""Extended admin routes for dashboards and bulk operations"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/04_admin_routes.ipynb.

# %% auto 0
__all__ = ['admin_dashboard', 'admin_detail', 'admin_bulk_delete']

# %% ../nbs/04_admin_routes.ipynb 3
from fasthtml.common import *
from monsterui.all import *
from dataclasses import fields
from .permissions import require_role

# %% ../nbs/04_admin_routes.ipynb 5
def admin_dashboard(rt, models, path='/admin'):
    """Simple dashboard showing all models.
    
    Args:
        rt: FastHTML route object
        models: List of tuples (name, table, model_class)
        path: Dashboard path (default: '/admin')
    """
    @rt(path)
    def get(req, sess):
        if not require_role('admin', req, sess):
            return RedirectResponse('/login', status_code=303)
        
        cards = []
        for name, table, _ in models:
            count = len(table())
            cards.append(
                Card(
                    H3(name.title()),
                    P(f"{count} items"),
                    A("Manage →", href=f"/admin/{name}/")
                )
            )
        
        return Container(
            H1("Admin Dashboard"),
            Grid(*cards, cols=3)
        )

# %% ../nbs/04_admin_routes.ipynb 7
def admin_detail(rt, name, table, model_class):
    """Show single item details.
    
    Args:
        rt: FastHTML route object
        name: Model name (e.g., 'product')
        table: Database table
        model_class: Dataclass model
    """
    @rt(f"/admin/{name}/{{id}}")
    def get(req, sess, id: int):
        if not require_role('admin', req, sess):
            return RedirectResponse('/login', status_code=303)
        
        item = table[id]
        field_divs = []
        
        for field in fields(model_class):
            field_divs.append(
                Div(
                    Label(field.name.replace('_', ' ').title()),
                    P(str(getattr(item, field.name, '')))
                )
            )
        
        return Container(
            H1(f"{name.title()} Details"),
            Card(*field_divs),
            A("Edit", href=f"/admin/{name}/{item.id}/edit"),
            A("Back", href=f"/admin/{name}/")
        )

# %% ../nbs/04_admin_routes.ipynb 9
def admin_bulk_delete(rt, name, table):
    """Delete multiple items at once.
    
    Args:
        rt: FastHTML route object
        name: Model name (e.g., 'product')
        table: Database table
    """
    @rt(f"/admin/{name}/delete", methods=['POST'])
    async def post(req, sess):
        if not require_role('admin', req, sess):
            return RedirectResponse('/login', status_code=303)
        
        form = await req.form()
        selected = form.getlist('selected')
        
        for item_id in selected:
            try:
                table.delete(int(item_id))
            except:
                pass  # Ignore errors for individual items
        
        # Redirect back to list view
        return RedirectResponse(f"/admin/{name}/", status_code=303)
